"""
Configs migration

Generation datetime: 2024-10-11 11:58:40.852471
"""

from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "6864cd9c"
DOWN_REVISION = "3d7b4e91"


def downgradeSettings(newSetting: dict) -> dict:
    newSetting.pop("usage_aggregation", None)
    newSetting.pop("http_port", None)
    newSetting.pop("db_user", None)
    newSetting.pop("db_password", None)
    newSetting.pop("db_name", None)
    oldSetting = dict(**newSetting)
    if oldSetting.get("storage_type") == "clickhouse":
        oldSetting["organization"] = newSetting.get("organization", "luna")
        oldSetting["token"] = newSetting.get("token", "secret-token")
        oldSetting["bucket"] = newSetting.get("bucket", "luna_monitoring")
    oldSetting["storage_type"] = "influx"
    return oldSetting


def upgradeSettings(setting: dict) -> dict:
    setting["usage_aggregation"] = 1
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_MONITORING",
        oldValidationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": False,
                    "description": "Influx monitoring settings.",
                }
            ]
        },
        newValidationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                        "usage_aggregation": {"type": "integer", "enum": [0, 1]},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                        "usage_aggregation": 1,
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": True,
                    "description": "Influx monitoring settings.",
                },
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["clickhouse"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "http_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "db_user": {"type": "string"},
                        "db_password": {"type": "string"},
                        "db_name": {"type": "string"},
                        "usage_aggregation": {"type": "integer", "enum": [0, 1]},
                    },
                    "default": {
                        "storage_type": "clickhouse",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 9090,
                        "http_port": 8123,
                        "db_user": "luna",
                        "db_password": "secret-password",
                        "db_name": "luna_monitoring",
                        "usage_aggregation": 1,
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "db_user",
                        "db_password",
                        "db_name",
                    ],
                    "type": "object",
                    "additionalProperties": True,
                    "description": "Clickhouse monitoring settings.",
                },
            ]
        },
        oldDefaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        newDefaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
            "usage_aggregation": 1,
        },
        settingUpgradeFunction=upgradeSettings,
        settingDowngradeFunction=downgradeSettings,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
    ),
]

externalLimitationsToModify: List[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)
