"""
Configs migration

Generation datetime: 2025-06-17 07:47:17.670387+00:00
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "14f7b4b6"
DOWN_REVISION = "6864cd9c"


def upgradeSettings(setting: dict) -> dict:
    setting.pop("usage_aggregation", None)
    return setting


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_MONITORING",
        oldValidationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                        "usage_aggregation": {"type": "integer", "enum": [0, 1]},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                        "usage_aggregation": 1,
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": True,
                    "description": "Influx monitoring settings.",
                },
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["clickhouse"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "http_port": {
                            "type": "integer",
                            "minimum": 1,
                            "maximum": 65535,
                        },
                        "db_user": {"type": "string"},
                        "db_password": {"type": "string"},
                        "db_name": {"type": "string"},
                        "usage_aggregation": {"type": "integer", "enum": [0, 1]},
                    },
                    "default": {
                        "storage_type": "clickhouse",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 9090,
                        "http_port": 8123,
                        "db_user": "luna",
                        "db_password": "secret-password",
                        "db_name": "luna_monitoring",
                        "usage_aggregation": 1,
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "db_user",
                        "db_password",
                        "db_name",
                    ],
                    "type": "object",
                    "additionalProperties": True,
                    "description": "Clickhouse monitoring settings.",
                },
            ]
        },
        newValidationSchema={
            "oneOf": [
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["influx"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "organization": {"type": "string"},
                        "token": {"type": "string"},
                        "bucket": {"type": "string"},
                    },
                    "default": {
                        "storage_type": "influx",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 8086,
                        "organization": "luna",
                        "token": "secret-token",
                        "bucket": "luna_monitoring",
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "organization",
                        "token",
                        "bucket",
                    ],
                    "type": "object",
                    "additionalProperties": True,
                    "description": "Influx monitoring settings.",
                },
                {
                    "properties": {
                        "storage_type": {"type": "string", "enum": ["clickhouse"]},
                        "send_data_for_monitoring": {"type": "integer", "enum": [0, 1]},
                        "use_ssl": {"type": "integer", "enum": [0, 1]},
                        "flushing_period": {"type": "number"},
                        "host": {"type": "string"},
                        "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                        "http_port": {
                            "type": "integer",
                            "minimum": 1,
                            "maximum": 65535,
                        },
                        "db_user": {"type": "string"},
                        "db_password": {"type": "string"},
                        "db_name": {"type": "string"},
                    },
                    "default": {
                        "storage_type": "clickhouse",
                        "send_data_for_monitoring": 1,
                        "use_ssl": 0,
                        "flushing_period": 1,
                        "host": "127.0.0.1",
                        "port": 9090,
                        "http_port": 8123,
                        "db_user": "luna",
                        "db_password": "secret-password",
                        "db_name": "luna_monitoring",
                    },
                    "required": [
                        "storage_type",
                        "send_data_for_monitoring",
                        "host",
                        "port",
                        "db_user",
                        "db_password",
                        "db_name",
                    ],
                    "type": "object",
                    "additionalProperties": True,
                    "description": "Clickhouse monitoring settings.",
                },
            ]
        },
        oldDefaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
            "usage_aggregation": 1,
        },
        newDefaultValue={
            "storage_type": "influx",
            "send_data_for_monitoring": 1,
            "use_ssl": 0,
            "flushing_period": 1,
            "host": "127.0.0.1",
            "port": 8086,
            "bucket": "luna_monitoring",
            "organization": "luna",
            "token": "secret-token",
        },
        settingUpgradeFunction=upgradeSettings,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-agent-unit",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-streams-retranslator",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
        newServices=[
            "luna-accounts",
            "luna-admin",
            "luna-api",
            "luna-backport3",
            "luna-backport4",
            "luna-events",
            "luna-faces",
            "luna-handlers",
            "luna-image-store",
            "luna-lambda",
            "luna-lambda-agent-unit",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
            "luna-licenses",
            "luna-matcher-proxy",
            "luna-python-matcher",
            "luna-remote-sdk",
            "luna-sender",
            "luna-streams-retranslator",
            "luna-tasks",
            "luna-video-agent",
            "luna-video-manager",
        ],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="MONITORING_AGGREGATION",
        validationSchema={"type": "boolean"},
        defaultValue=True,
        description="enable monitoring data aggregation",
        services=["luna-admin"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="MONITORING_AGGREGATION")
